﻿using System.Collections;
using System.Windows;
using System.Windows.Controls;
using DragDropSample.ViewModels;
using Telerik.Windows.Controls;
//NOTE: We need this namespace.
using Telerik.Windows.Controls.DragDrop;
using System;

namespace DragDropSample
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            RadDragAndDropManager.DragStartThreshold = 2;

            InitializeComponent();

            listBox.ItemsSource = MyViewModel.GetItems("ListBox Item");
            gridView.ItemsSource = MyViewModel.GetItems("GridView Item");

            //There are two sides of a DragDrop, a source side which "supplies"
            //the payload and a destination side which can accept the payload 
            //(and the DragDrop). There are also two types of events. Events that 
            //ask us if something should be possible and events that notify us
            //that somethinng that just happened (Query and Info events respectively). 

            //Here we add handlers for the source side
            RadDragAndDropManager.AddDragQueryHandler(this, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
            RadDragAndDropManager.AddDragInfoHandler(this, new EventHandler<DragDropEventArgs>(OnDragInfo));

            //And the handlers for the destination side.
            RadDragAndDropManager.AddDropQueryHandler(this, OnDropQuery);
            RadDragAndDropManager.AddDropInfoHandler(this, OnDropInfo);
        }

        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            //Again, we handle the drop query at a root leve, but we could
            //add the handler to a specific control if we want more granularity.
            if (e.Options.Status == DragStatus.DropDestinationQuery)
            {
                //We would only check that the source and the destination are
                //different
                e.QueryResult = e.Options.Destination != e.Options.Source;
                e.Handled = true;
            }
        }

        private void OnDropInfo(object sender, DragDropEventArgs e)
        {
            //Here we can change the drag cue to let the user know what
            //action is possible. We could change the background of the
            //arrow if it is in use.
            if (e.Options.Status == DragStatus.DropPossible)
            {
                //Drop is possible, change the content of the DragCue or 
                //notify the user that a drag is possible.
            }

            if (e.Options.Status == DragStatus.DropImpossible)
            {
                //Drop is not possible. Here generally we will revert the changes
                //done in DropPossible.
            }

            if (e.Options.Status == DragStatus.DropComplete)
            {
                //Here we can generally check the payload and add handle the case
                //when it is an IEnumerable (when multiple items are dragged).
                //The paylaod could also be a custom class that contains mroe info
                //for the DragDrop. HEre we know that it always is an item that we can 
                //display. We just add the item here, we do not interact with the
                //drag source. Removing the item (if at all removed should happen OnDragIndo
                //so that there will be separation of handling the DragDrop, which is more flexible.
                IList itemsSource = null;
                if (e.Options.Destination is System.Windows.Controls.ListBox)
                {
                    itemsSource = (e.Options.Destination as System.Windows.Controls.ListBox).ItemsSource as IList;
                }
                else
                {
                    itemsSource = (e.Options.Destination as RadGridView).ItemsSource as IList;
                }

                itemsSource.Add(e.Options.Payload);
            }
        }

        private void OnDragInfo(object sender, DragDropEventArgs e)
        {
            //We want to remove the item on a successful drop. But we have added
            //this event handler to the root visual, we have to then check the 
            //drag source here. Alternatively we can add separates handlers to 
            //the different controls.
            if (e.Options.Status == DragStatus.DragComplete)
            {
                IList itemsSource = null;
                if (e.Options.Destination is System.Windows.Controls.ListBox)
                {
                    itemsSource = (e.Options.Destination as System.Windows.Controls.ListBox).ItemsSource as IList;
                }
                else
                {
                    itemsSource = (e.Options.Destination as RadGridView).ItemsSource as IList;
                }

                itemsSource.Remove(e.Options.Payload);
            }
        }

        private void OnDragQuery(object sender, DragDropQueryEventArgs e)
        {
            //We are asked here if the item should be dragged:
            if (e.Options.Status == DragStatus.DragQuery)
            {
                //The answer is yes, but we can get e.Options.Source and 
                //check the item for something and only then allow it.
                e.QueryResult = true;

                //Generally it is a good idea to handle routed events if 
                //we are taking some action.
                e.Handled = true;

                //We want to make sure that the payload is something maningful that can
                //be used by the destination side. We could simply add the dragged item (or items)
                //or we could create a custom class that will provide more information in
                //a complex app.
                //Here we get the ViewModel iitem. We could get the selected items of a cotnrol
                //for multiple items DragDrop.
                e.Options.Payload = (e.Source as FrameworkElement).DataContext;

                //We can decide to add a drag cue here
                //to help the user. We can add any ContentControl as a
                //drag cue but we can also generate an arrow because it is easier:
                e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
            }

            //The question here is if the source item can  be dropped,
            //again we can say yes/no depending on some rules.
            if (e.Options.Status == DragStatus.DropSourceQuery)
            {
                e.QueryResult = true;
                e.Handled = true;
            }
        }



    }
}
